@@ -16,7 +16,7 @@ set -o errexit
 save_pwd="$(pwd)"
 
 . ./libmongocrypt/.evergreen/setup-env.sh
-MONGO_C_DRIVER_VERSION=1.16.0-pre . ./libmongocrypt/.evergreen/prep_c_driver_source.sh
+. ./libmongocrypt/.evergreen/prep_c_driver_source.sh
 . ./libmongocrypt/.evergreen/build_all.sh
 
 cd ${save_pwd}
