@@ -17,8 +17,8 @@
 /* Needed for strptime */
 #define _GNU_SOURCE
 
-#include "src/kms_message/kms_message.h"
-#include "src/kms_message_private.h"
+#include "kms_message/kms_message.h"
+#include "kms_message_private.h"
 
 #ifndef _WIN32
 #include <dirent.h>
@@ -32,35 +32,13 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <time.h>
-#include <src/kms_message/kms_b64.h>
-#include <src/hexlify.h>
-#include <src/kms_request_str.h>
-#include <src/kms_kv_list.h>
-#include <src/kms_port.h>
+#include "kms_message/kms_b64.h"
+#include "hexlify.h"
+#include "kms_request_str.h"
+#include "kms_kv_list.h"
+#include "kms_port.h"
 
-#define ASSERT_CONTAINS(_a, _b)                                              \
-   do {                                                                      \
-      kms_request_str_t *_a_str = kms_request_str_new_from_chars ((_a), -1); \
-      kms_request_str_t *_b_str = kms_request_str_new_from_chars ((_b), -1); \
-      kms_request_str_t *_a_lower = kms_request_str_new ();                  \
-      kms_request_str_t *_b_lower = kms_request_str_new ();                  \
-      kms_request_str_append_lowercase (_a_lower, (_a_str));                 \
-      kms_request_str_append_lowercase (_b_lower, (_b_str));                 \
-      if (NULL == strstr ((_a_lower->str), (_b_lower->str))) {               \
-         fprintf (stderr,                                                    \
-                  "%s:%d %s(): [%s] does not contain [%s]\n",                \
-                  __FILE__,                                                  \
-                  __LINE__,                                                  \
-                  __FUNCTION__,                                              \
-                  _a,                                                        \
-                  _b);                                                       \
-         abort ();                                                           \
-      }                                                                      \
-      kms_request_str_destroy (_a_str);                                      \
-      kms_request_str_destroy (_b_str);                                      \
-      kms_request_str_destroy (_a_lower);                                    \
-      kms_request_str_destroy (_b_lower);                                    \
-   } while (0)
+#include "test_kms_assert.h"
 
 const char *aws_test_suite_dir = "aws-sig-v4-test-suite";
 
@@ -323,57 +301,6 @@ read_req (const char *path)
    return request;
 }
 
-ssize_t
-first_non_matching (const char *x, const char *y)
-{
-   size_t len = strlen (x) > strlen (y) ? strlen (x) : strlen (y);
-   size_t i;
-
-   for (i = 0; i < len; i++) {
-      if (x[i] != y[i]) {
-         return i;
-      }
-   }
-
-   if (strlen (x) > strlen (y)) {
-      return strlen (y) + 1;
-   }
-
-   if (strlen (y) > strlen (x)) {
-      return strlen (x) + 1;
-   }
-
-   /* the strings match */
-   return -1;
-}
-
-void
-compare_strs (const char *test_name, const char *expect, const char *actual)
-{
-   if (0 != strcmp (actual, expect)) {
-      fprintf (stderr,
-               "%s failed, mismatch starting at %zd\n"
-               "--- Expect (%zu chars) ---\n%s\n"
-               "--- Actual (%zu chars) ---\n%s\n",
-               test_name,
-               first_non_matching (expect, actual),
-               strlen (expect),
-               expect,
-               strlen (actual),
-               actual);
-
-      abort ();
-   }
-}
-
-#define ASSERT_CMPSTR(_a, _b) compare_strs (__FUNCTION__, (_a), (_b))
-
-#define ASSERT(stmt)                                    \
-   if (!(stmt)) {                                       \
-      fprintf (stderr, "statement failed %s\n", #stmt); \
-      abort ();                                         \
-   }
-
 void
 test_compare (kms_request_t *request,
               char *(*func) (kms_request_t *),
@@ -386,7 +313,7 @@ test_compare (kms_request_t *request,
 
    expect = read_test (dir_path, suffix);
    actual = func (request);
-   compare_strs (test_name, expect, actual);
+   ASSERT_CMPSTR (expect, actual);
    free (actual);
    free (expect);
    free (test_name);
@@ -503,7 +430,7 @@ example_signature_test (void)
 
    KMS_ASSERT (kms_request_get_signing_key (request, signing));
    sig = hexlify (signing, 32);
-   compare_strs (__FUNCTION__, expect, sig);
+   ASSERT_CMPSTR (expect, sig);
    free (sig);
    kms_request_destroy (request);
 }
@@ -557,7 +484,7 @@ path_normalization_test (void)
       in = kms_request_str_new_from_chars (test[0], -1);
       out = test[1];
       norm = kms_request_str_path_normalized (in);
-      compare_strs (__FUNCTION__, out, norm->str);
+      ASSERT_CMPSTR (out, norm->str);
       kms_request_str_destroy (in);
       kms_request_str_destroy (norm);
    }
@@ -1161,6 +1088,20 @@ kms_signature_test (void)
    KMS_ASSERT (!ret);
 }
 
+static void
+kms_request_kmip_prohibited_test (void)
+{
+   kms_request_opt_t *opt;
+   kms_request_t *req;
+
+   opt = kms_request_opt_new ();
+   kms_request_opt_set_provider (opt, KMS_REQUEST_PROVIDER_KMIP);
+   req = kms_request_new ("method", "path_and_query", opt);
+   ASSERT_REQUEST_ERROR (req, "Function not applicable to KMIP");
+   kms_request_destroy (req);
+   kms_request_opt_destroy (opt);
+}
+
 #define RUN_TEST(_func)                                          \
    do {                                                          \
       if (!selector || 0 == kms_strcasecmp (#_func, selector)) { \
@@ -1170,6 +1111,25 @@ kms_signature_test (void)
       }                                                          \
    } while (0)
 
+extern void kms_kmip_writer_test (void);
+extern void kms_kmip_reader_test (void);
+extern void kms_kmip_reader_negative_int_test (void);
+extern void kms_kmip_reader_find_test (void);
+extern void kms_kmip_reader_find_and_recurse_test (void);
+extern void kms_kmip_reader_find_and_read_enum_test (void);
+extern void kms_kmip_reader_find_and_read_bytes_test (void);
+extern void kms_kmip_request_register_secretdata_test (void);
+extern void kms_kmip_request_register_secretdata_invalid_test (void);
+extern void kms_kmip_request_get_test (void);
+extern void kms_kmip_request_activate_test (void);
+extern void kms_kmip_response_parser_test (void);
+extern void kms_kmip_response_get_unique_identifier_test (void);
+extern void kms_kmip_response_get_secretdata_test (void);
+extern void kms_kmip_response_get_secretdata_notfound_test (void);
+extern void kms_kmip_response_parser_reuse_test (void);
+extern void kms_kmip_response_parser_excess_test (void);
+extern void kms_kmip_response_parser_notenough_test (void);
+
 int
 main (int argc, char *argv[])
 {
@@ -1215,6 +1175,27 @@ main (int argc, char *argv[])
 
    RUN_TEST (kms_signature_test);
 
+   RUN_TEST (kms_kmip_writer_test);
+   RUN_TEST (kms_kmip_reader_test);
+   RUN_TEST (kms_kmip_reader_negative_int_test);
+   RUN_TEST (kms_kmip_reader_test);
+   RUN_TEST (kms_kmip_reader_find_and_recurse_test);
+   RUN_TEST (kms_kmip_reader_find_and_read_enum_test);
+   RUN_TEST (kms_kmip_reader_find_and_read_bytes_test);
+   RUN_TEST (kms_kmip_request_register_secretdata_test);
+   RUN_TEST (kms_kmip_request_register_secretdata_invalid_test);
+   RUN_TEST (kms_kmip_request_get_test);
+   RUN_TEST (kms_kmip_request_activate_test);
+   RUN_TEST (kms_request_kmip_prohibited_test);
+   RUN_TEST (kms_kmip_response_parser_test);
+   RUN_TEST (kms_kmip_response_get_unique_identifier_test);
+   RUN_TEST (kms_kmip_response_get_secretdata_test);
+   RUN_TEST (kms_kmip_response_get_secretdata_notfound_test);
+   RUN_TEST (kms_kmip_response_parser_reuse_test);
+   RUN_TEST (kms_kmip_response_parser_excess_test);
+   RUN_TEST (kms_kmip_response_parser_notenough_test);
+
+
    if (!ran_tests) {
       KMS_ASSERT (argc == 2);
       fprintf (stderr, "No such test: \"%s\"\n", argv[1]);
