@@ -22,6 +22,7 @@
 #include "mongocrypt-kms-ctx-private.h"
 #include "mongocrypt-opts-private.h"
 #include "mongocrypt-status-private.h"
+#include "mongocrypt-util-private.h"
 #include <kms_message/kms_b64.h>
 #include <kms_message/kms_azure_request.h>
 #include <kms_message/kms_gcp_request.h>
@@ -38,6 +39,8 @@ typedef struct {
  */
 #define DEFAULT_MAX_KMS_BYTE_REQUEST 1024
 #define SHA256_LEN 32
+#define DEFAULT_HTTPS_PORT "443"
+#define DEFAULT_KMIP_PORT "5696"
 
 static bool
 _sha256 (void *ctx, const char *input, size_t len, unsigned char *hash_out)
@@ -105,12 +108,24 @@ _set_kms_crypto_hooks (_mongocrypt_crypto_t *crypto,
    }
 }
 
+static bool
+is_kms (_kms_request_type_t kms_type)
+{
+   return kms_type == MONGOCRYPT_KMS_KMIP_REGISTER ||
+          kms_type == MONGOCRYPT_KMS_KMIP_ACTIVATE ||
+          kms_type == MONGOCRYPT_KMS_KMIP_GET;
+}
+
 static void
 _init_common (mongocrypt_kms_ctx_t *kms,
               _mongocrypt_log_t *log,
               _kms_request_type_t kms_type)
 {
-   kms->parser = kms_response_parser_new ();
+   if (is_kms (kms_type)) {
+      kms->parser = kms_kmip_response_parser_new (NULL /* reserved */);
+   } else {
+      kms->parser = kms_response_parser_new ();
+   }
    kms->log = log;
    kms->status = mongocrypt_status_new ();
    kms->req_type = kms_type;
@@ -177,6 +192,12 @@ _mongocrypt_kms_ctx_init_aws_decrypt (mongocrypt_kms_ctx_t *kms,
    kms_request_opt_destroy (opt);
    kms_request_set_service (kms->req, "kms");
 
+   if (crypt_opts->kms_provider_aws.session_token) {
+      kms_request_add_header_field (kms->req,
+                                    "X-Amz-Security-Token",
+                                    crypt_opts->kms_provider_aws.session_token);
+   }
+
    if (kms_request_get_error (kms->req)) {
       CLIENT_ERR ("error constructing KMS message: %s",
                   kms_request_get_error (kms->req));
@@ -232,6 +253,7 @@ _mongocrypt_kms_ctx_init_aws_decrypt (mongocrypt_kms_ctx_t *kms,
       kms->endpoint = bson_strdup_printf ("kms.%s.amazonaws.com",
                                           key->kek.provider.aws.region);
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    ret = true;
 done:
@@ -306,6 +328,12 @@ _mongocrypt_kms_ctx_init_aws_encrypt (
    kms_request_opt_destroy (opt);
    kms_request_set_service (kms->req, "kms");
 
+   if (crypt_opts->kms_provider_aws.session_token) {
+      kms_request_add_header_field (kms->req,
+                                    "X-Amz-Security-Token",
+                                    crypt_opts->kms_provider_aws.session_token);
+   }
+
    if (kms_request_get_error (kms->req)) {
       CLIENT_ERR ("error constructing KMS message: %s",
                   kms_request_get_error (kms->req));
@@ -361,6 +389,7 @@ _mongocrypt_kms_ctx_init_aws_encrypt (
       kms->endpoint = bson_strdup_printf ("kms.%s.amazonaws.com",
                                           ctx_opts->kek.provider.aws.region);
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    ret = true;
 done:
@@ -375,7 +404,7 @@ mongocrypt_kms_ctx_bytes_needed (mongocrypt_kms_ctx_t *kms)
    if (!kms) {
       return 0;
    }
-   /* TODO: an oddity of kms-message. After retrieving the JSON result, it
+   /* TODO: an oddity of kms-message. After retrieving the result, it
     * resets the parser. */
    if (!mongocrypt_status_ok (kms->status) ||
        !_mongocrypt_buffer_empty (&kms->result)) {
@@ -745,6 +774,84 @@ fail:
    return ret;
 }
 
+static bool
+_ctx_done_kmip_register (mongocrypt_kms_ctx_t *kms_ctx)
+{
+   kms_response_t *res = NULL;
+   mongocrypt_status_t *status = kms_ctx->status;
+   bool ret = false;
+   char *uid;
+
+   res = kms_response_parser_get_response (kms_ctx->parser);
+   if (!res) {
+      CLIENT_ERR ("Error getting KMIP response: %s",
+                  kms_response_parser_error (kms_ctx->parser));
+      goto done;
+   }
+
+   uid = kms_kmip_response_get_unique_identifier (res);
+   if (!uid) {
+      CLIENT_ERR (
+         "Error getting UniqueIdentifer from KMIP Register response: %s",
+         kms_response_get_error (res));
+      goto done;
+   }
+
+   if (!_mongocrypt_buffer_steal_from_string (&kms_ctx->result, uid)) {
+      CLIENT_ERR ("Error storing KMS UniqueIdentifer result");
+      bson_free (uid);
+      goto done;
+   }
+   ret = true;
+
+done:
+   kms_response_destroy (res);
+   return ret;
+}
+
+static bool
+_ctx_done_kmip_activate (mongocrypt_kms_ctx_t *kms_ctx)
+{
+   return _ctx_done_kmip_register (kms_ctx);
+}
+
+static bool
+_ctx_done_kmip_get (mongocrypt_kms_ctx_t *kms_ctx)
+{
+   kms_response_t *res = NULL;
+   mongocrypt_status_t *status = kms_ctx->status;
+   bool ret = false;
+   uint8_t *secretdata;
+   size_t secretdata_len;
+
+   res = kms_response_parser_get_response (kms_ctx->parser);
+   if (!res) {
+      CLIENT_ERR ("Error getting KMIP response: %s",
+                  kms_response_parser_error (kms_ctx->parser));
+      goto done;
+   }
+
+   secretdata = kms_kmip_response_get_secretdata (res, &secretdata_len);
+   if (!secretdata) {
+      CLIENT_ERR ("Error getting SecretData from KMIP Get response: %s",
+                  kms_response_get_error (res));
+      goto done;
+   }
+
+   if (!_mongocrypt_buffer_steal_from_data_and_size (
+          &kms_ctx->result, secretdata, secretdata_len)) {
+      CLIENT_ERR ("Error storing KMS SecretData result");
+      bson_free (secretdata);
+      goto done;
+   }
+
+   ret = true;
+
+done:
+   kms_response_destroy (res);
+   return ret;
+}
+
 bool
 mongocrypt_kms_ctx_feed (mongocrypt_kms_ctx_t *kms, mongocrypt_binary_t *bytes)
 {
@@ -780,32 +887,47 @@ mongocrypt_kms_ctx_feed (mongocrypt_kms_ctx_t *kms, mongocrypt_binary_t *bytes)
    }
 
    if (!kms_response_parser_feed (kms->parser, bytes->data, bytes->len)) {
-      CLIENT_ERR ("KMS response parser error with status %d, error: '%s'",
-                  kms_response_parser_status (kms->parser),
-                  kms_response_parser_error (kms->parser));
+      if (is_kms (kms->req_type)) {
+         /* The KMIP response parser does not suport kms_response_parser_status.
+          * Only report the error string. */
+         CLIENT_ERR ("KMS response parser error with error: '%s'",
+                     kms_response_parser_error (kms->parser));
+      } else {
+         CLIENT_ERR ("KMS response parser error with status %d, error: '%s'",
+                     kms_response_parser_status (kms->parser),
+                     kms_response_parser_error (kms->parser));
+      }
+
       return false;
    }
 
    if (0 == mongocrypt_kms_ctx_bytes_needed (kms)) {
-      if (kms->req_type == MONGOCRYPT_KMS_AWS_ENCRYPT) {
-         return _ctx_done_aws (kms, "CiphertextBlob");
-      } else if (kms->req_type == MONGOCRYPT_KMS_AWS_DECRYPT) {
-         return _ctx_done_aws (kms, "Plaintext");
-      } else if (kms->req_type == MONGOCRYPT_KMS_AZURE_OAUTH) {
-         return _ctx_done_oauth (kms);
-      } else if (kms->req_type == MONGOCRYPT_KMS_AZURE_WRAPKEY) {
-         return _ctx_done_azure_wrapkey_unwrapkey (kms);
-      } else if (kms->req_type == MONGOCRYPT_KMS_AZURE_UNWRAPKEY) {
-         return _ctx_done_azure_wrapkey_unwrapkey (kms);
-      } else if (kms->req_type == MONGOCRYPT_KMS_GCP_OAUTH) {
-         return _ctx_done_oauth (kms);
-      } else if (kms->req_type == MONGOCRYPT_KMS_GCP_ENCRYPT) {
-         return _ctx_done_gcp (kms, "ciphertext");
-      } else if (kms->req_type == MONGOCRYPT_KMS_GCP_DECRYPT) {
-         return _ctx_done_gcp (kms, "plaintext");
-      } else {
+      switch (kms->req_type) {
+      default:
          CLIENT_ERR ("Unknown request type");
          return false;
+      case MONGOCRYPT_KMS_AWS_ENCRYPT:
+         return _ctx_done_aws (kms, "CiphertextBlob");
+      case MONGOCRYPT_KMS_AWS_DECRYPT:
+         return _ctx_done_aws (kms, "Plaintext");
+      case MONGOCRYPT_KMS_AZURE_OAUTH:
+         return _ctx_done_oauth (kms);
+      case MONGOCRYPT_KMS_AZURE_WRAPKEY:
+         return _ctx_done_azure_wrapkey_unwrapkey (kms);
+      case MONGOCRYPT_KMS_AZURE_UNWRAPKEY:
+         return _ctx_done_azure_wrapkey_unwrapkey (kms);
+      case MONGOCRYPT_KMS_GCP_OAUTH:
+         return _ctx_done_oauth (kms);
+      case MONGOCRYPT_KMS_GCP_ENCRYPT:
+         return _ctx_done_gcp (kms, "ciphertext");
+      case MONGOCRYPT_KMS_GCP_DECRYPT:
+         return _ctx_done_gcp (kms, "plaintext");
+      case MONGOCRYPT_KMS_KMIP_REGISTER:
+         return _ctx_done_kmip_register (kms);
+      case MONGOCRYPT_KMS_KMIP_ACTIVATE:
+         return _ctx_done_kmip_activate (kms);
+      case MONGOCRYPT_KMS_KMIP_GET:
+         return _ctx_done_kmip_get (kms);
       }
    }
    return true;
@@ -921,7 +1043,7 @@ _mongocrypt_kms_ctx_init_azure_auth (mongocrypt_kms_ctx_t *kms,
    mongocrypt_status_t *status;
    _mongocrypt_endpoint_t *identity_platform_endpoint;
    char *scope = NULL;
-   const char *host;
+   const char *hostname;
    char *request_string;
    bool ret = false;
 
@@ -932,11 +1054,12 @@ _mongocrypt_kms_ctx_init_azure_auth (mongocrypt_kms_ctx_t *kms,
 
    if (identity_platform_endpoint) {
       kms->endpoint = bson_strdup (identity_platform_endpoint->host_and_port);
-      host = identity_platform_endpoint->host;
+      hostname = identity_platform_endpoint->host;
    } else {
       kms->endpoint = bson_strdup ("login.microsoftonline.com");
-      host = kms->endpoint;
+      hostname = "login.microsoftonline.com";
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    if (key_vault_endpoint) {
       /* Request a custom scope. It is URL encoded, like
@@ -953,7 +1076,7 @@ _mongocrypt_kms_ctx_init_azure_auth (mongocrypt_kms_ctx_t *kms,
    kms_request_opt_set_connection_close (opt, true);
    kms_request_opt_set_provider (opt, KMS_REQUEST_PROVIDER_AZURE);
    kms->req =
-      kms_azure_request_oauth_new (host,
+      kms_azure_request_oauth_new (hostname,
                                    scope,
                                    crypt_opts->kms_provider_azure.tenant_id,
                                    crypt_opts->kms_provider_azure.client_id,
@@ -1006,6 +1129,7 @@ _mongocrypt_kms_ctx_init_azure_wrapkey (
 
    kms->endpoint = bson_strdup (
       ctx_opts->kek.provider.azure.key_vault_endpoint->host_and_port);
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
    host = ctx_opts->kek.provider.azure.key_vault_endpoint->host;
 
    opt = kms_request_opt_new ();
@@ -1068,6 +1192,7 @@ _mongocrypt_kms_ctx_init_azure_unwrapkey (mongocrypt_kms_ctx_t *kms,
 
    kms->endpoint =
       bson_strdup (key->kek.provider.azure.key_vault_endpoint->host_and_port);
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
    host = key->kek.provider.azure.key_vault_endpoint->host;
 
    opt = kms_request_opt_new ();
@@ -1155,7 +1280,7 @@ _mongocrypt_kms_ctx_init_gcp_auth (mongocrypt_kms_ctx_t *kms,
    _mongocrypt_endpoint_t *auth_endpoint;
    char *scope = NULL;
    char *audience = NULL;
-   const char *host;
+   const char *hostname;
    char *request_string;
    bool ret = false;
    ctx_with_status_t ctx_with_status;
@@ -1168,13 +1293,14 @@ _mongocrypt_kms_ctx_init_gcp_auth (mongocrypt_kms_ctx_t *kms,
 
    if (auth_endpoint) {
       kms->endpoint = bson_strdup (auth_endpoint->host_and_port);
-      host = auth_endpoint->host;
+      hostname = auth_endpoint->host;
       audience = bson_strdup_printf ("https://%s/token", auth_endpoint->host);
    } else {
       kms->endpoint = bson_strdup ("oauth2.googleapis.com");
-      host = kms->endpoint;
+      hostname = "oauth2.googleapis.com";
       audience = bson_strdup_printf ("https://oauth2.googleapis.com/token");
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    if (kms_endpoint) {
       /* Request a custom scope. */
@@ -1193,7 +1319,7 @@ _mongocrypt_kms_ctx_init_gcp_auth (mongocrypt_kms_ctx_t *kms,
          opt, _sign_rsaes_pkcs1_v1_5_trampoline, &ctx_with_status);
    }
    kms->req = kms_gcp_request_oauth_new (
-      host,
+      hostname,
       crypt_opts->kms_provider_gcp.email,
       audience,
       scope,
@@ -1241,7 +1367,7 @@ _mongocrypt_kms_ctx_init_gcp_encrypt (
    mongocrypt_status_t *status;
    char *path_and_query = NULL;
    char *payload = NULL;
-   const char *host;
+   const char *hostname;
    char *request_string;
    bool ret = false;
    char *bearer_token_value = NULL;
@@ -1252,18 +1378,19 @@ _mongocrypt_kms_ctx_init_gcp_encrypt (
    if (ctx_opts->kek.provider.gcp.endpoint) {
       kms->endpoint =
          bson_strdup (ctx_opts->kek.provider.gcp.endpoint->host_and_port);
-      host = ctx_opts->kek.provider.gcp.endpoint->host;
+      hostname = ctx_opts->kek.provider.gcp.endpoint->host;
    } else {
       kms->endpoint = bson_strdup ("cloudkms.googleapis.com");
-      host = kms->endpoint;
+      hostname = "cloudkms.googleapis.com";
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    opt = kms_request_opt_new ();
    BSON_ASSERT (opt);
    kms_request_opt_set_connection_close (opt, true);
    kms_request_opt_set_provider (opt, KMS_REQUEST_PROVIDER_GCP);
    kms->req =
-      kms_gcp_request_encrypt_new (host,
+      kms_gcp_request_encrypt_new (hostname,
                                    access_token,
                                    ctx_opts->kek.provider.gcp.project_id,
                                    ctx_opts->kek.provider.gcp.location,
@@ -1311,7 +1438,7 @@ _mongocrypt_kms_ctx_init_gcp_decrypt (mongocrypt_kms_ctx_t *kms,
    mongocrypt_status_t *status;
    char *path_and_query = NULL;
    char *payload = NULL;
-   const char *host;
+   const char *hostname;
    char *request_string;
    bool ret = false;
    char *bearer_token_value = NULL;
@@ -1322,17 +1449,18 @@ _mongocrypt_kms_ctx_init_gcp_decrypt (mongocrypt_kms_ctx_t *kms,
    if (key->kek.provider.gcp.endpoint) {
       kms->endpoint =
          bson_strdup (key->kek.provider.gcp.endpoint->host_and_port);
-      host = key->kek.provider.gcp.endpoint->host;
+      hostname = key->kek.provider.gcp.endpoint->host;
    } else {
       kms->endpoint = bson_strdup ("cloudkms.googleapis.com");
-      host = kms->endpoint;
+      hostname = "cloudkms.googleapis.com";
    }
+   _mongocrypt_apply_default_port (&kms->endpoint, DEFAULT_HTTPS_PORT);
 
    opt = kms_request_opt_new ();
    BSON_ASSERT (opt);
    kms_request_opt_set_connection_close (opt, true);
    kms_request_opt_set_provider (opt, KMS_REQUEST_PROVIDER_GCP);
-   kms->req = kms_gcp_request_decrypt_new (host,
+   kms->req = kms_gcp_request_decrypt_new (hostname,
                                            access_token,
                                            key->kek.provider.gcp.project_id,
                                            key->kek.provider.gcp.location,
@@ -1366,4 +1494,149 @@ fail:
    bson_free (payload);
    bson_free (bearer_token_value);
    return ret;
-}
\ No newline at end of file
+}
+
+bool
+_mongocrypt_kms_ctx_init_kmip_register (mongocrypt_kms_ctx_t *kms_ctx,
+                                        const _mongocrypt_endpoint_t *endpoint,
+                                        const uint8_t *secretdata,
+                                        uint32_t secretdata_len,
+                                        _mongocrypt_log_t *log)
+{
+   mongocrypt_status_t *status;
+   bool ret = false;
+   const uint8_t *reqdata;
+   size_t reqlen;
+
+   _init_common (kms_ctx, log, MONGOCRYPT_KMS_KMIP_REGISTER);
+   status = kms_ctx->status;
+
+   kms_ctx->endpoint = bson_strdup (endpoint->host_and_port);
+   _mongocrypt_apply_default_port (&kms_ctx->endpoint, DEFAULT_KMIP_PORT);
+   kms_ctx->req = kms_kmip_request_register_secretdata_new (
+      NULL /* reserved */, secretdata, secretdata_len);
+
+   if (!kms_ctx->req) {
+      CLIENT_ERR ("Error creating KMIP register request: %s",
+                  kms_request_get_error (kms_ctx->req));
+      goto done;
+   }
+
+   reqdata = kms_request_to_bytes (kms_ctx->req, &reqlen);
+   if (!_mongocrypt_buffer_copy_from_data_and_size (
+          &kms_ctx->msg, reqdata, reqlen)) {
+      CLIENT_ERR ("Error storing KMS request payload");
+      goto done;
+   }
+
+   ret = true;
+done:
+   return ret;
+}
+
+bool
+_mongocrypt_kms_ctx_init_kmip_activate (mongocrypt_kms_ctx_t *kms_ctx,
+                                        const _mongocrypt_endpoint_t *endpoint,
+                                        const char *unique_identifier,
+                                        _mongocrypt_log_t *log)
+{
+   mongocrypt_status_t *status;
+   bool ret = false;
+   size_t reqlen;
+   const uint8_t *reqdata;
+
+   _init_common (kms_ctx, log, MONGOCRYPT_KMS_KMIP_ACTIVATE);
+   status = kms_ctx->status;
+
+   kms_ctx->endpoint = bson_strdup (endpoint->host_and_port);
+   _mongocrypt_apply_default_port (&kms_ctx->endpoint, DEFAULT_KMIP_PORT);
+   kms_ctx->req =
+      kms_kmip_request_activate_new (NULL /* reserved */, unique_identifier);
+
+   if (!kms_ctx->req) {
+      CLIENT_ERR ("Error creating KMIP activate request: %s",
+                  kms_request_get_error (kms_ctx->req));
+      goto done;
+   }
+
+   reqdata = kms_request_to_bytes (kms_ctx->req, &reqlen);
+   if (!_mongocrypt_buffer_copy_from_data_and_size (
+          &kms_ctx->msg, reqdata, reqlen)) {
+      CLIENT_ERR ("Error storing KMS request payload");
+      goto done;
+   }
+
+   ret = true;
+done:
+   return ret;
+}
+
+bool
+_mongocrypt_kms_ctx_init_kmip_get (mongocrypt_kms_ctx_t *kms_ctx,
+                                   const _mongocrypt_endpoint_t *endpoint,
+                                   const char *unique_identifier,
+                                   _mongocrypt_log_t *log)
+{
+   mongocrypt_status_t *status;
+   bool ret = false;
+   size_t reqlen;
+   const uint8_t *reqdata;
+
+   _init_common (kms_ctx, log, MONGOCRYPT_KMS_KMIP_GET);
+   status = kms_ctx->status;
+
+   kms_ctx->endpoint = bson_strdup (endpoint->host_and_port);
+   _mongocrypt_apply_default_port (&kms_ctx->endpoint, DEFAULT_KMIP_PORT);
+   kms_ctx->req =
+      kms_kmip_request_get_new (NULL /* reserved */, unique_identifier);
+
+   if (!kms_ctx->req) {
+      CLIENT_ERR ("Error creating KMIP get request: %s",
+                  kms_request_get_error (kms_ctx->req));
+      goto done;
+   }
+
+   reqdata = kms_request_to_bytes (kms_ctx->req, &reqlen);
+   if (!_mongocrypt_buffer_copy_from_data_and_size (
+          &kms_ctx->msg, reqdata, reqlen)) {
+      CLIENT_ERR ("Error storing KMS request payload");
+      goto done;
+   }
+
+   ret = true;
+done:
+   return ret;
+}
+
+const char *
+set_and_ret (const char *what, uint32_t *len)
+{
+   if (len) {
+      BSON_ASSERT (size_to_uint32 (strlen (what), len));
+   }
+   return what;
+}
+
+const char *
+mongocrypt_kms_ctx_get_kms_provider (mongocrypt_kms_ctx_t *kms, uint32_t *len)
+{
+   switch (kms->req_type) {
+   default:
+      BSON_ASSERT ("unknown KMS request type");
+   case MONGOCRYPT_KMS_AWS_ENCRYPT:
+   case MONGOCRYPT_KMS_AWS_DECRYPT:
+      return set_and_ret ("aws", len);
+   case MONGOCRYPT_KMS_AZURE_OAUTH:
+   case MONGOCRYPT_KMS_AZURE_WRAPKEY:
+   case MONGOCRYPT_KMS_AZURE_UNWRAPKEY:
+      return set_and_ret ("azure", len);
+   case MONGOCRYPT_KMS_GCP_OAUTH:
+   case MONGOCRYPT_KMS_GCP_ENCRYPT:
+   case MONGOCRYPT_KMS_GCP_DECRYPT:
+      return set_and_ret ("gcp", len);
+   case MONGOCRYPT_KMS_KMIP_REGISTER:
+   case MONGOCRYPT_KMS_KMIP_ACTIVATE:
+   case MONGOCRYPT_KMS_KMIP_GET:
+      return set_and_ret ("kmip", len);
+   }
+}
