@@ -492,11 +492,12 @@ fail:
 
 /* Create a TLS stream to a host. */
 static mongoc_stream_t *
-connect_with_tls (const char *endpoint,
-                  int32_t connecttimeoutms,
-                  bson_error_t *error)
+connect_stream_with_tls (mongoc_ssl_opt_t *ssl_opt,
+                         const char *endpoint,
+                         int32_t connecttimeoutms,
+                         bson_error_t *error)
 {
-   mongoc_stream_t *stream;
+   mongoc_stream_t *stream = NULL;
    mongoc_socket_t *sock = NULL;
    struct addrinfo hints;
    struct addrinfo *result, *rp;
@@ -563,8 +564,7 @@ done:
       freeaddrinfo (result);
    }
    if (success) {
-      stream = mongoc_stream_tls_new_with_hostname (
-         stream, host, (mongoc_ssl_opt_t *) mongoc_ssl_opt_get_default (), 1);
+      stream = mongoc_stream_tls_new_with_hostname (stream, host, ssl_opt, 1);
    } else {
       mongoc_stream_destroy (stream);
       stream = NULL;
@@ -643,6 +643,7 @@ _state_need_kms (_state_machine_t *state_machine, bson_error_t *error)
    mongocrypt_binary_t *http_reply = NULL;
    const char *endpoint;
    uint32_t sockettimeout;
+   mongoc_ssl_opt_t ssl_opt;
 
    sockettimeout = MONGOC_DEFAULT_SOCKETTIMEOUTMS;
    kms_ctx = mongocrypt_ctx_next_kms_ctx (state_machine->ctx);
@@ -661,11 +662,16 @@ _state_need_kms (_state_machine_t *state_machine, bson_error_t *error)
          goto fail;
       }
 
-      tls_stream = connect_with_tls (endpoint, sockettimeout, error);
+      ssl_opt = *mongoc_ssl_opt_get_default ();
+      ssl_opt.ca_file = state_machine->tls_ca_file;
+      ssl_opt.pem_file = state_machine->tls_certificate_key_file;
+      tls_stream =
+         connect_stream_with_tls (&ssl_opt, endpoint, sockettimeout, error);
 #ifdef MONGOC_ENABLE_SSL_SECURE_CHANNEL
       /* Retry once with schannel as a workaround for CDRIVER-3566. */
       if (!tls_stream) {
-         tls_stream = connect_with_tls (endpoint, sockettimeout, error);
+         tls_stream =
+            connect_stream_with_tls (&ssl_opt, endpoint, sockettimeout, error);
       }
 #endif
       if (!tls_stream) {
@@ -676,8 +682,9 @@ _state_need_kms (_state_machine_t *state_machine, bson_error_t *error)
       iov.iov_len = mongocrypt_binary_len (http_req);
 
       if (state_machine->trace) {
-         MONGOC_DEBUG (
-            "--> sending KMS message: \n%.*s", (int) iov.iov_len, (char*) iov.iov_base);
+         MONGOC_DEBUG ("--> sending KMS message: \n%.*s",
+                       (int) iov.iov_len,
+                       (char *) iov.iov_base);
       }
 
       if (!_mongoc_stream_writev_full (
